<?php

namespace Boomee\Influencer\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Boomee\Influencer\Http\Requests\InfluencerRequest;
use Boomee\Influencer\Interfaces\InfluencerRepositoryInterface;
use Boomee\Influencer\Models\Influencer;
use Boomee\Influencer\Forms\Influencer as Form;

/**
 * APIController  class for influencer.
 */
class InfluencerAPIController extends BaseController
{

    /**
     * Initialize influencer resource controller.
     *
     * @param type InfluencerRepositoryInterface $influencer
     *
     * @return null
     */
    public function __construct(InfluencerRepositoryInterface $influencer)
    {
        parent::__construct();
        $this->repository = $influencer;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Boomee\Influencer\Repositories\Criteria\InfluencerResourceCriteria::class);
    }

    /**
     * Display a list of influencer.
     *
     * @return Response
     */
    public function index(InfluencerRequest $request)
    {
        return $this->repository
            ->setPresenter(\Boomee\Influencer\Repositories\Presenter\InfluencerPresenter::class)
            ->paginate();
    }

    /**
     * Display influencer.
     *
     * @param Request $request
     * @param Model   $influencer
     *
     * @return Response
     */
    public function show(InfluencerRequest $request, Influencer $influencer)
    {
        return $influencer->setPresenter(\Boomee\Influencer\Repositories\Presenter\InfluencerListPresenter::class);
        ;
    }

    /**
     * Create new influencer.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InfluencerRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('influencer::influencer.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('influencer/influencer/' . $influencer->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('influencer/influencer');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the influencer.
     *
     * @param Request $request
     * @param Model   $influencer
     *
     * @return Response
     */
    public function update(InfluencerRequest $request, Influencer $influencer)
    {
        try {
            $data = $request->all();

            $influencer->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('influencer::influencer.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('influencer/influencer/' . $influencer->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('influencer/influencer/' . $influencer->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the influencer.
     *
     * @param Model   $influencer
     *
     * @return Response
     */
    public function destroy(InfluencerRequest $request, Influencer $influencer)
    {
        try {
            $influencer->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('influencer::influencer.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('influencer/influencer/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('influencer/influencer/' . $influencer->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
