<?php

namespace Boomee\Influencer\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Boomee\Influencer\Http\Requests\InfluencerRequest;
use Boomee\Influencer\Interfaces\InfluencerRepositoryInterface;
use Boomee\Influencer\Models\Influencer;

/**
 * Resource controller class for influencer.
 */
class InfluencerResourceController extends BaseController
{

    /**
     * Initialize influencer resource controller.
     *
     * @param type InfluencerRepositoryInterface $influencer
     *
     * @return null
     */
    public function __construct(InfluencerRepositoryInterface $influencer)
    {
        parent::__construct();
        $this->repository = $influencer;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Boomee\Influencer\Repositories\Criteria\InfluencerResourceCriteria::class);
    }

    /**
     * Display a list of influencer.
     *
     * @return Response
     */
    public function index(InfluencerRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Boomee\Influencer\Repositories\Presenter\InfluencerPresenter::class)
                ->$function();
        }

        $influencers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('influencer::influencer.names'))
            ->view('influencer::influencer.index', true)
            ->data(compact('influencers', 'view'))
            ->output();
    }

    /**
     * Display influencer.
     *
     * @param Request $request
     * @param Model   $influencer
     *
     * @return Response
     */
    public function show(InfluencerRequest $request, Influencer $influencer)
    {

        if ($influencer->exists) {
            $view = 'influencer::influencer.show';
        } else {
            $view = 'influencer::influencer.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('influencer::influencer.name'))
            ->data(compact('influencer'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new influencer.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InfluencerRequest $request)
    {

        $influencer = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('influencer::influencer.name')) 
            ->view('influencer::influencer.create', true) 
            ->data(compact('influencer'))
            ->output();
    }

    /**
     * Create new influencer.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InfluencerRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $influencer                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('influencer::influencer.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('influencer/influencer/' . $influencer->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/influencer/influencer'))
                ->redirect();
        }

    }

    /**
     * Show influencer for editing.
     *
     * @param Request $request
     * @param Model   $influencer
     *
     * @return Response
     */
    public function edit(InfluencerRequest $request, Influencer $influencer)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('influencer::influencer.name'))
            ->view('influencer::influencer.edit', true)
            ->data(compact('influencer'))
            ->output();
    }

    /**
     * Update the influencer.
     *
     * @param Request $request
     * @param Model   $influencer
     *
     * @return Response
     */
    public function update(InfluencerRequest $request, Influencer $influencer)
    {
        try {
            $attributes = $request->all();

            $influencer->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('influencer::influencer.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('influencer/influencer/' . $influencer->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('influencer/influencer/' . $influencer->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the influencer.
     *
     * @param Model   $influencer
     *
     * @return Response
     */
    public function destroy(InfluencerRequest $request, Influencer $influencer)
    {
        try {

            $influencer->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('influencer::influencer.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('influencer/influencer/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('influencer/influencer/' . $influencer->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple influencer.
     *
     * @param Model   $influencer
     *
     * @return Response
     */
    public function delete(InfluencerRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('influencer::influencer.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('influencer/influencer'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/influencer/influencer'))
                ->redirect();
        }

    }

    /**
     * Restore deleted influencers.
     *
     * @param Model   $influencer
     *
     * @return Response
     */
    public function restore(InfluencerRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('influencer::influencer.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/influencer/influencer'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/influencer/influencer/'))
                ->redirect();
        }

    }

}
