<?php

namespace Boomee\Influencer\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Boomee\Influencer\Http\Requests\SocialRequest;
use Boomee\Influencer\Interfaces\SocialRepositoryInterface;
use Boomee\Influencer\Models\Social;

/**
 * Resource controller class for social.
 */
class SocialResourceController extends BaseController
{

    /**
     * Initialize social resource controller.
     *
     * @param type SocialRepositoryInterface $social
     *
     * @return null
     */
    public function __construct(SocialRepositoryInterface $social)
    {
        parent::__construct();
        $this->repository = $social;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Boomee\Influencer\Repositories\Criteria\SocialResourceCriteria::class);
    }

    /**
     * Display a list of social.
     *
     * @return Response
     */
    public function index(SocialRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Boomee\Influencer\Repositories\Presenter\SocialPresenter::class)
                ->$function();
        }

        $socials = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('influencer::social.names'))
            ->view('influencer::social.index', true)
            ->data(compact('socials', 'view'))
            ->output();
    }

    /**
     * Display social.
     *
     * @param Request $request
     * @param Model   $social
     *
     * @return Response
     */
    public function show(SocialRequest $request, Social $social)
    {

        if ($social->exists) {
            $view = 'influencer::social.show';
        } else {
            $view = 'influencer::social.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('influencer::social.name'))
            ->data(compact('social'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new social.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SocialRequest $request)
    {

        $social = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('influencer::social.name')) 
            ->view('influencer::social.create', true) 
            ->data(compact('social'))
            ->output();
    }

    /**
     * Create new social.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SocialRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $social                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('influencer::social.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('influencer/social/' . $social->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/influencer/social'))
                ->redirect();
        }

    }

    /**
     * Show social for editing.
     *
     * @param Request $request
     * @param Model   $social
     *
     * @return Response
     */
    public function edit(SocialRequest $request, Social $social)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('influencer::social.name'))
            ->view('influencer::social.edit', true)
            ->data(compact('social'))
            ->output();
    }

    /**
     * Update the social.
     *
     * @param Request $request
     * @param Model   $social
     *
     * @return Response
     */
    public function update(SocialRequest $request, Social $social)
    {
        try {
            $attributes = $request->all();

            $social->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('influencer::social.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('influencer/social/' . $social->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('influencer/social/' . $social->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the social.
     *
     * @param Model   $social
     *
     * @return Response
     */
    public function destroy(SocialRequest $request, Social $social)
    {
        try {

            $social->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('influencer::social.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('influencer/social/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('influencer/social/' . $social->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple social.
     *
     * @param Model   $social
     *
     * @return Response
     */
    public function delete(SocialRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('influencer::social.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('influencer/social'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/influencer/social'))
                ->redirect();
        }

    }

    /**
     * Restore deleted socials.
     *
     * @param Model   $social
     *
     * @return Response
     */
    public function restore(SocialRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('influencer::social.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/influencer/social'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/influencer/social/'))
                ->redirect();
        }

    }

}
